import { autocompletion, pickedCompletion, } from '@codemirror/autocomplete';
import { CompletionItemKind, CompletionRequest, InsertReplaceEdit, TextEdit, } from 'vscode-languageserver-protocol';
import { clientFacet, fileUriFacet } from './client';
import { textDocumentField } from './textDocumentSync';
import { Facet } from '@codemirror/state';
export const lspComplete = (overrides = {}) => autocompletion({
    activateOnTyping: true,
    override: [complete],
    maxRenderedOptions: 20,
    ...overrides,
});
export const infoRendererFacet = Facet.define({
    static: true,
    combine: (values) => { var _a; return (_a = values[0]) !== null && _a !== void 0 ? _a : undefined; },
});
export async function complete(context) {
    // TODO does this work well with translations and stuff?
    const word = context.matchBefore(/\w*/);
    if (!word)
        return null;
    const client = context.state.facet(clientFacet.reader);
    const fileUri = context.state.facet(fileUriFacet.reader);
    const infoRenderer = context.state.facet(infoRendererFacet.reader);
    const textDocument = context.state.field(textDocumentField);
    const results = await client.sendRequest(CompletionRequest.type, {
        textDocument: { uri: fileUri },
        position: textDocument.positionAt(context.pos),
    });
    // No results
    if (results === null)
        return null;
    // CompletionList not supported
    if (!Array.isArray(results))
        return null;
    return {
        from: word.from,
        options: results.map((completionItem) => {
            var _a;
            return ({
                label: (_a = completionItem.insertText) !== null && _a !== void 0 ? _a : completionItem.label,
                displayLabel: completionItem.label,
                apply: hasApplicableTextEdit(completionItem)
                    ? (view, completion) => applyEdit(view, completion, completionItem, textDocument)
                    : undefined,
                type: convertLSPKindToCodeMirrorKind(completionItem.kind),
                info: infoRenderer ? (_) => infoRenderer(completionItem) : undefined,
            });
        }),
    };
}
function hasApplicableTextEdit(completionItem) {
    return (!!completionItem.textEdit &&
        (TextEdit.is(completionItem.textEdit) || InsertReplaceEdit.is(completionItem.textEdit)));
}
var CMCompletionType;
(function (CMCompletionType) {
    CMCompletionType["Class"] = "class";
    CMCompletionType["Constant"] = "constant";
    CMCompletionType["Enum"] = "enum";
    CMCompletionType["Function"] = "function";
    CMCompletionType["Interface"] = "interface";
    CMCompletionType["Keyword"] = "keyword";
    CMCompletionType["Method"] = "method";
    CMCompletionType["Namespace"] = "namespace";
    CMCompletionType["Property"] = "property";
    CMCompletionType["Text"] = "text";
    CMCompletionType["Type"] = "type";
    CMCompletionType["Variable"] = "variable";
})(CMCompletionType || (CMCompletionType = {}));
function convertLSPKindToCodeMirrorKind(kind) {
    if (!kind)
        return CMCompletionType.Text;
    switch (kind) {
        case CompletionItemKind.Constructor:
        case CompletionItemKind.Class:
            return CMCompletionType.Class;
        case CompletionItemKind.Constant:
            return CMCompletionType.Constant;
        case CompletionItemKind.Enum:
        case CompletionItemKind.EnumMember:
            return CMCompletionType.Enum;
        case CompletionItemKind.Snippet:
        case CompletionItemKind.Function:
            return CMCompletionType.Function;
        case CompletionItemKind.Interface:
            return CMCompletionType.Interface;
        case CompletionItemKind.Operator:
        case CompletionItemKind.Keyword:
            return CMCompletionType.Keyword;
        case CompletionItemKind.Method:
            return CMCompletionType.Method;
        case CompletionItemKind.File:
        case CompletionItemKind.Folder:
        case CompletionItemKind.Module:
            return CMCompletionType.Namespace;
        case CompletionItemKind.Property:
        case CompletionItemKind.Reference:
        case CompletionItemKind.Field:
            return CMCompletionType.Property;
        case CompletionItemKind.Struct:
        case CompletionItemKind.TypeParameter:
            return CMCompletionType.Type;
        case CompletionItemKind.Event:
        case CompletionItemKind.Value:
        case CompletionItemKind.Variable:
            return CMCompletionType.Variable;
        case CompletionItemKind.Color:
        case CompletionItemKind.Unit:
        case CompletionItemKind.Text:
        default:
            return CMCompletionType.Text;
    }
}
function applyEdit(view, completion, item, textDocument) {
    const { textEdit } = item;
    let start = 0;
    let end = 0;
    let newText = '';
    if (TextEdit.is(textEdit)) {
        start = textDocument.offsetAt(textEdit.range.start);
        end = textDocument.offsetAt(textEdit.range.end);
        newText = textEdit.newText;
    }
    else if (InsertReplaceEdit.is(textEdit)) {
        start = textDocument.offsetAt(textEdit.replace.start);
        end = textDocument.offsetAt(textEdit.replace.end);
        newText = textEdit.newText;
    }
    view.dispatch({
        // Tell the completion engine which item we chose
        annotations: [pickedCompletion.of(completion)],
        // Move cursor to after the text
        selection: { anchor: start + newText.length, head: start + newText.length },
        // Apply the text edit
        changes: view.state.changes({
            from: start,
            to: end,
            insert: newText,
        }),
    });
}
//# sourceMappingURL=complete.js.map